from parserutils import generateLogicalLines, maskStringsAndComments, maskStringsAndRemoveComments
import re

TABWIDTH = 4

class StructuralNode:
    def __init__(self):
        self.childNodes = []
        self._parent = None

    def addChild(self, node):
        self.childNodes.append(node)
        node.setParent(self)

    def setParent(self, parent):
        self._parent = parent
        #print "setParent",self,parent.fqn
        if parent.fqn == "":
            self.setFQN(self.name)
        else:
            self.setFQN(parent.fqn+"."+self.name)

    def setFQN(self, fqn):
        self.fqn = fqn
        for child in self.getChildNodes():  # fqn has changed - tell children
            child.setParent(self)

    def getParent(self):
        return self._parent

    def getChildNodes(self):
        return self.childNodes

    # badly named: actually returns line numbers of import statements
    def getImportLines(self):
        try:
            return self.importlines
        except AttributeError:
            return[]

    def getLinesNotIncludingThoseBelongingToChildScopes(self):
        #src = self.module.getSourceNode().getLines()
        srclines = self.module.getMaskedLines()
        lines = []
        lineno = self.getStartLine()
        for child in self.getChildNodes():
            lines+=srclines[lineno-1: child.getStartLine()-1]
            lineno = child.getEndLine()
        lines+=srclines[lineno-1: self.getEndLine()-1]
        return lines


class Module(StructuralNode):
    def __init__(self, name, maskedsrc):
        StructuralNode.__init__(self)
        self.name = name
        self.fqn = name  # overriden later if added to a tree
        self.indent = -TABWIDTH
        self.flattenedNodes = []
        self.maskedsrc = maskedsrc
        self._physicallines = None
        self.module = self


    def getMaskedLines(self):
        if self._physicallines is None:
            # make sure src is actually masked
            # (could just have keywords masked)
            self.maskedsrc = maskStringsAndComments(self.maskedsrc)
            self._physicallines = self.maskedsrc.splitlines(1)
        return self._physicallines

    def getFlattenedListOfChildNodes(self):
        return self.flattenedNodes

    def getStartLine(self):
        return 1

    def getEndLine(self):
        return len(self.getMaskedLines())+1


    def getSourceNode(self):
        return self.sourcenode

    def setSourceNode(self, sourcenode):
        self.sourcenode = sourcenode


indentRE = re.compile("^(\s*)\S")
class Node:
    # module = the module node
    # linenum = starting line number
    def __init__(self, name, module, linenum, indent):
        self.name = name
        self.module = module
        self.linenum = linenum
        self.endline = None
        self.indent = indent

    def getStartLine(self):
        return self.linenum

    def getEndLine(self):
        if self.endline is None:
            physicallines = self.module.getMaskedLines()
            lineno = self.linenum
            logicallines = generateLogicalLines(physicallines[lineno-1:])

            # skip the first line, because it's the declaration
            line = logicallines.next()
            lineno+=line.count("\n")

            # scan to the end of the fn
            for line in logicallines:
                #print lineno,":",line,
                match = indentRE.match(line)
                if match and match.end()-1 <= self.indent:
                    break
                lineno+=line.count("\n")
            self.endline = lineno
        return self.endline




baseClassesRE = re.compile("class\s+[^(]+\((.+)\):")

class Class(StructuralNode, Node):
    def __init__(self, name, module, linenum, indent):
        StructuralNode.__init__(self)
        Node.__init__(self, name, module, linenum, indent)
        self.type = "Class"

    def getBaseClassNames(self):
        match = baseClassesRE.search(self.getLine(self.getStartLine()))
        if match:
            return [s.strip()for s in match.group(1).split(",")]
        else:
            return []

    def getLine(self, linenum):
        return self.module.getSourceNode().getLines()[linenum-1]


class Function(StructuralNode, Node):
    def __init__(self, name, module, linenum, indent):
        StructuralNode.__init__(self)
        Node.__init__(self, name, module, linenum, indent)
        self.type = "Function"
