# This module (hopefully) supercedes load_application

# A lot of this code is take from Pythius -
# Copyright (GPL) 2001 Jrgen Hermann <jh@web.de>

import os
import constants
from extended_ast import*
from bike import log
from parser import ParserError

def containsAny(str, set):
    """ Check whether 'str' contains ANY of the chars in 'set'
    """
    return 1in[c in str for c in set]


def _visit_pyfiles(list, dirname, names):
    """ Helper for getFilesForName().
    """

    # get extension for python source files
    if not globals().has_key('_py_ext'):
        import imp
        global _py_ext
        _py_ext = [triple[0]for triple in imp.get_suffixes()if triple[2] == imp.PY_SOURCE][0]

    # don't recurse into CVS directories
    if 'CVS'in names:
        names.remove('CVS')

    for n in names:
        if os.path.isdir(os.path.join(dirname, n))and \
           not os.path.exists(os.path.join(dirname, n, "__init__.py")):
            names.remove(n)

    # add all *.py files to list
    list.extend(
    [os.path.join(dirname, file)
    for file in names
    if os.path.splitext(file)[1] == _py_ext])


def _get_modpkg_path(dotted_name, pathlist = None):
    """ Get the filesystem path for a module or a package.

        Return the file system path to a file for a module,
        and to a directory for a package. Return None if
        the name is not found, or is a builtin or extension module.
    """
    import imp

    # split off top-most name
    parts = dotted_name.split('.', 1)

    if len(parts) > 1:
        # we have a dotted path, import top-level package
        try:
            file, pathname, description = imp.find_module(parts[0], pathlist)
            if file: file.close()
        except ImportError:
            return None

        # check if it's indeed a package
        if description[2] == imp.PKG_DIRECTORY:
            # recursively handle the remaining name parts
            pathname = _get_modpkg_path(parts[1], [pathname])
        else:
            pathname = None
    else:
        # plain name
        try:
            file, pathname, description = imp.find_module(dotted_name, pathlist)
            if file: file.close()
            if description[2]not in[imp.PY_SOURCE, imp.PKG_DIRECTORY]:
                pathname = None
        except ImportError:
            pathname = None

    return pathname


def getFilesForName(name):
    """ Get a list of module files for a filename, a module or package name,
        or a directory.
    """
    import imp

    if not os.path.exists(name):
        # check for glob chars
        if containsAny(name, "*?[]"):
            import glob
            files = glob.glob(name)
            list = []
            for file in files:
                list.extend(getFilesForName(file))
            return list

        # try to find module or package
        name = _get_modpkg_path(name)
        if not name:
            return[]

    if os.path.isdir(name):
        # find all python files in directory
        list = []
        os.path.walk(name, _visit_pyfiles, list)
        return list
    elif os.path.exists(name):
        # a single file
        return[name]

    return[]


def loadFile(filename):
    f = open(filename)
    src = f.read()
    f.close()
    return src


# returns the directory which holds the first package of the package
# hierarchy under which 'filename' belongs
def getRootDirectory(filename):
    if os.path.isdir(filename):
        dir = filename
    else:
        dir = os.path.dirname(filename)
    while dir != ""and \
          os.path.exists(os.path.join(dir, "__init__.py")):
        dir = os.path.dirname(dir)
    return dir

def filenameToModulePath(directoriesPreceedingRoot, fname):
    import os
    # strip off directories preceeding root package directory
    if directoriesPreceedingRoot != "":
        mpath = fname.replace(directoriesPreceedingRoot, "")
    else:
        mpath = fname

    if(mpath[0] == os.path.normpath("/")):
        mpath = mpath[1:]
    mpath, ext = os.path.splitext(mpath)
    mpath = mpath.replace(os.path.normpath("/"), ".")
    return mpath

def getModuleName(modulepath):
    a = modulepath.rfind(".")
    if a == -1:
        return modulepath
    return modulepath[a+1:]

def _findOrCreatePackage(root, modpath):
    node = root
    # find or create package
    path = ""
    for packagename in modpath.split(".")[:-1]:
        if path == "":
            path = packagename
        else:
            path+="."+packagename

        try:
            node = node.getPackage(packagename)
        except NoSuchPackageException:
            node.addPackage(Package(path, packagename))
            node = node.getPackage(packagename)
    return node


def load(path, root = None):
    if root == None:
        root = Root([], [], [])

    directoriesPreceedingRoot = getRootDirectory(path)

    files = getFilesForName(path)
    for fname in files:
        modpath = filenameToModulePath(directoriesPreceedingRoot, fname)
        print >> log.progress, "Parsing",modpath
        packagenode = _findOrCreatePackage(root, modpath)
        modname = getModuleName(modpath)
        if packagenode.hasSource(modname):
            src = loadFile(fname)
            packagenode.getSource(modname).reload(src)
        else:
            src = loadFile(fname)
            try:
                packagenode.addSource(Source.createFromFile(fname, modname))
            except ParserError:
                print "Warning: Couldnt parse",fname
                pass   # skip it
    return root

if __name__ == "__main__":
    simulateLoad("/usr/local/lib/python2.2")
