#!/usr/bin/env python 
import setpath
import unittest
import os
from bike.parsing.extended_ast import Root, Source

from test_load import tmpfile, writeTmpTestFile

from bike import testdata
from bike.mock import Mock
from output import*
from bike.testutils import*


testdata_class = """
class foo:
    pass
"""


testdata_comment_ending_in_newline = """
# comment
"""

testdata_comment_not_ending_in_newline = """
pass # comment
"""


class TestSave(unittest.TestCase):
    def testSave_doesntWriteFilesIfSourceNotModified(self):
        writeTmpTestFile("notpass")
        s = Source.createFromString(tmpfile, "foobahfile", "pass")
        xast = Root([], [s], "")
        save(xast)  # setModified() not called on source!
        open(tmpfile).read() == "notpass"  # file hasnt changed
        deleteTmpTestFile()


    def testSave_writesFilesIfSourceModified(self):
        try:
            writeTmpTestFile("notpass")
            s = Source.createFromString(tmpfile, "foobahfile", "pass")
            xast = Root([], [s], "")
            s.setModified()
            filesWritten = save(xast)
            assert open(tmpfile).read() == "pass"
            assert filesWritten == [os.path.abspath(tmpfile)]
        finally:
            deleteTmpTestFile()




if __name__ == "__main__":
    unittest.main()
