#!/usr/bin/env python
import setpath
import unittest
from bike.parsing.undo import*
from bike import testdata
from bike.testutils import*
from bike.parsing.tokenutils import createToken
import tokenize

class TestUndoStack(unittest.TestCase):
    def test_undoesText(self):
        u = getUndoStack()
        source = createAST(testdata.class_and_method)
        srcbefore = source.getSourceAsString()

        tokens = source.getTokens()
        tokens.insertAndPositionIteratorOnNewElement(tokens.begin(), createToken(tokenize.NL, ""))

        u.commitUndoFrame()
        assert srcbefore != source.getSourceAsString()
        u.undo()
        assert srcbefore == source.getSourceAsString()

    def test_addingSourceASecondTimeDoesntGetStored(self):
        u = getUndoStack()
        source = createAST(testdata.class_and_method)
        srcbefore = source.getSourceAsString()

        tokens = source.getTokens()
        tokens.insertAndPositionIteratorOnNewElement(tokens.begin(), createToken(tokenize.NL, ""))

        tokens = source.getTokens()

        u.commitUndoFrame()
        assert srcbefore != source.getSourceAsString()
        u.undo()
        assert srcbefore == source.getSourceAsString()

    def test_oldFramesAreClippedWhenReachesUndoBufferSize(self):
        u = getUndoStack()
        u.setUndoBufferSize(2)
        source = createAST(testdata.class_and_method)

        srcbefore = source.getSourceAsString()
        tokens = source.getTokens()
        tokens.insertAndPositionIteratorOnNewElement(tokens.begin(), createToken(tokenize.NL, ""))
        u.commitUndoFrame()

        src2 = source.getSourceAsString()
        source._source = src2   # fix the source - this is a hack
        tokens = source.getTokens()
        tokens.insertAndPositionIteratorOnNewElement(tokens.begin(), createToken(tokenize.NL, ""))
        u.commitUndoFrame()

        src3 = source.getSourceAsString()
        source._source = src3   # fix the source - this is a hack
        tokens = source.getTokens()
        tokens.insertAndPositionIteratorOnNewElement(tokens.begin(), createToken(tokenize.NL, ""))

        u.commitUndoFrame()

        assert src3 != source.getSourceAsString()
        u.undo()
        assert src3 == source.getSourceAsString()
        u.undo()
        assert src2 == source.getSourceAsString()
        caught = 0
        try:
            u.undo()
        except UndoStackEmptyException:
            caught = 1
        assert caught

if __name__ == "__main__":
    unittest.main()
