# queries to do with module/class/function relationships
from getTypeOf import getTypeOf, getTypeOfExpr
from bike.parsing.visitor import walk
from bike import log

# first name is a child of scope
# (e.g. a method in a class, a nested function in a function, a class in a module)
def getChildByName(scope,name):
    matches = [c for c in scope.getChildNodes() if c.name == name]
    if matches != []:
        return matches[0]



def getAllClassesInHierarchy(root,classobj):
    rootClasses = _getRootClasses(classobj)
    print >>log.progress,"Getting root classes"
    relatedClasses = [] + rootClasses
    print >>log.progress,"Getting root classes - done"
    for rootClass in rootClasses:
        relatedClasses += getAllDerivedClasses(rootClass,root)
    return relatedClasses

def _getRootClasses(klass):
    if klass is None:  # i.e. dont have base class in our ast
        return None
    if klass.getBaseClassNames() == []:  # i.e. is a root class
        return[klass]
    else:
        rootclasses = []
        for base in klass.getBaseClassNames():
            baseclass = getTypeOf(klass,base)
            rootclass = _getRootClasses(baseclass)
            if rootclass is None:  # base class not in our ast
                rootclass = [klass]
            rootclasses+=rootclass
        return rootclasses


def getAllDerivedClasses(baseclass, root, subclasses = []):
    class ClassVisitor:
        def visitSource(self,node):
            if node.containsString(baseclass.name):
                self.visit(node.fastparseroot)
            
        def visitClass(self, node):
            for basename in node.getBaseClassNames():
                if getTypeOf(node,basename) == baseclass:
                    subclasses.append(node)
                    getAllDerivedClasses(node,root,subclasses)
            for child in node.getChildNodes():
                self.visit(child)
                
    walk(root, ClassVisitor())
    return subclasses
