#!/usr/bin/env python
import setpath
import unittest
import os

from bike import testdata
from bike.query.findDefinition import findDefinitionByCoords
from bike.query.getTypeOf import getTypeOf
from bike.testutils import trimLines, createSourceNodeAt
from bike.parsing.extended_ast import Root

        
class TestFindDefinitionByCoords(unittest.TestCase):
    def test_findsClassRef(self):
        src=trimLines("""
        class TheClass:
            pass
        a = TheClass()
        """)
        root = createSourceNodeAt(Root(),src,"mymodule")
        module = getTypeOf(root,"mymodule")
        defn = [x for x in findDefinitionByCoords(module,3,6)]
        assert defn[0].filename == os.path.abspath("mymodule.py")
        assert defn[0].lineno == 1
        assert defn[0].confidence == 100

    def tests_findsMethodRef(self):
        src=trimLines("""
        class TheClass:
            def theMethod(self):
                pass
        a = TheClass()
        a.theMethod()
        """)
        root = createSourceNodeAt(Root(),src,"mymodule")
        module = getTypeOf(root,"mymodule")
        defn = [x for x in findDefinitionByCoords(module,5,3)]
        assert defn[0].filename == os.path.abspath("mymodule.py")
        assert defn[0].lineno == 2
        assert defn[0].confidence == 100
        

    def test_returnsOtherMethodsWithSameName(self):
        src=trimLines("""
        class TheClass:
            def theMethod(self):
                pass
        a = SomeOtherClass()
        a.theMethod()
        """)
        root = createSourceNodeAt(Root(),src,"mymodule")
        module = getTypeOf(root,"mymodule")
        defn = [x for x in findDefinitionByCoords(module,5,3)]
        assert defn[0].filename == os.path.abspath("mymodule.py")
        assert defn[0].lineno == 2
        assert defn[0].confidence == 50


    def test_findsClassRefUsingFromImportStatement(self):
        src=trimLines("""
        from a.b.bah import TheClass
        """)
        classsrc=trimLines("""
        class TheClass:
            pass
        """)
        root = createSourceNodeAt(Root(),src,"a.foo")
        root = createSourceNodeAt(root, classsrc, "a.b.bah")
        module = getTypeOf(root,"a.foo")
        defn = [x for x in findDefinitionByCoords(module,1,21)]
        assert defn[0].filename == os.path.abspath(os.path.join("a","b","bah.py"))
        assert defn[0].lineno == 1
        assert defn[0].confidence == 100


if __name__ == "__main__":
    unittest.main()
