#!/usr/bin/env python
import setpath
import unittest
from bike import testdata
from bike.testutils import trimLines,createSourceNodeAt
from bike.query.getTypeOf import getTypeOf, UnfoundType
from bike.parsing.extended_ast import Root, Instance
from bike.parsing.fastparserast import Class, Function

class TestGetTypeOf(unittest.TestCase):
    def test_getsTypeOfSimpleClassInstanceReference(self):
        src = trimLines("""
        from b.bah import TheClass
        a = TheClass()
        a.theMethod()
        """)
        root = createSourceNodeAt(Root(),src,"a.foo")
        root = createSourceNodeAt(root, testdata.TheClass, "a.b.bah")
        module = getTypeOf(root,"a.foo")
        res =  getTypeOf(module,"a")
        # put correct instance in here!
        assert isinstance(res,Instance)
        assert isinstance(res.getType(),Class)
        self.assertEqual(res.getType().fqn,"a.b.bah.TheClass")


    def test_getsTypeOfInstanceThatIsAnAttributeOfSelf(self):
        src = trimLines("""
        class TheClass:
            def theMethod(self):
                pass
        
        class AnotherClass:
            def __init__(self):
                self.a = TheClass()
            def anotherFn(self):
                self.a.theMethod()
        """)
        root = createSourceNodeAt(Root(),src,"a.foo")
        theclass = getTypeOf(root,"a.foo.TheClass")
        fn = getTypeOf(root,"a.foo.AnotherClass.anotherFn")
        self.assertEqual(getTypeOf(fn,"self.a").getType(), theclass)
        
        

    def test_doesntGetTypeDefinedInChildFunction(self):
        src = trimLines("""
        from b.bah import TheClass
        a = AnotherClass()
        def foo():
            a = TheClass()
        a.theMethod()
        """)
        root = createSourceNodeAt(Root(),src,"a.foo")
        root = createSourceNodeAt(root, testdata.TheClass, "a.b.bah")
        themodule = getTypeOf(root,"a.foo")
        #print getTypeOf(themodule,"a")
        assert isinstance(getTypeOf(themodule,"a"),UnfoundType)

    def test_getsTypeOfClassReferencedViaAlias(self):
        src = trimLines("""
        from b.bah import TheClass as FooBah
        FooBah()
        """)
        root = createSourceNodeAt(Root(),src,"a.foo")
        root = createSourceNodeAt(root, testdata.TheClass, "a.b.bah")
        themodule = getTypeOf(root,"a.foo")
        self.assertEqual(getTypeOf(themodule,"FooBah").fqn,
                         "a.b.bah.TheClass")
        

    def test_getsTypeOfClassImportedFromPackageScope(self):
        initfile = trimLines("""
        from bah import TheClass
        """)
        src = trimLines("""
        from a import b
        b.TheClass()
        """)
        root = createSourceNodeAt(Root(),src,"a.foo")
        root = createSourceNodeAt(root,initfile,"a.b.__init__")
        root = createSourceNodeAt(root, testdata.TheClass, "a.b.bah")
        themodule = getTypeOf(root,"a.foo")
        self.assertEqual(getTypeOf(themodule,"b.TheClass").fqn,
                         "a.b.bah.TheClass")

if __name__ == "__main__":
    unittest.main()
