#!/usr/bin/env python
import setpath
import unittest
import os
from bike import testdata
from bike.testutils import *
from bike.query.getTypeOf import getTypeOf
from bike.parsing.fastparserast import Module
from bike.query.relationships import getAllClassesInHierarchy, getAllDerivedClasses

class TestGetAllClassesInHierarchy(unittest.TestCase):

    def test_getsDerivedDerivedClass(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class DerivedClass(BaseClass):
            pass

        class DerivedDerivedClass(DerivedClass):
            def theMethod(self):
                print 'hello'
        """)
        root = createSourceNodeAt(Root(),src,"a.foo")
        root = createSourceNodeAt(root, testdata.TheClass, "a.b.bah")
        classobj = getTypeOf(root,"a.b.bah.TheClass")
        allclasses = getAllClassesInHierarchy(root,classobj)
        assert allclasses[0].fqn == "a.b.bah.TheClass"
        assert allclasses[1].fqn == "a.foo.DerivedClass"
        assert allclasses[2].fqn == "a.foo.DerivedDerivedClass"


class TestGetAllDerivedClasses(unittest.TestCase):
    def test_getsDerivedDerivedClass(self):
        src = trimLines("""
        from b.bah import TheClass as BaseClass
        
        class DerivedClass(BaseClass):
            pass

        class DerivedDerivedClass(DerivedClass):
            def theMethod(self):
                print 'hello'
        """)
        root = createSourceNodeAt(Root(),src,"a.foo")
        root = createSourceNodeAt(root, testdata.TheClass, "a.b.bah")
        classobj = getTypeOf(root,"a.b.bah.TheClass")
        derivedclasses = getAllDerivedClasses(root,classobj)
        assert derivedclasses[0].fqn == "a.foo.DerivedClass"
        assert derivedclasses[1].fqn == "a.foo.DerivedDerivedClass"


if __name__ == "__main__":
    unittest.main()
