import re
import compiler
from bike.parsing import visitor
from bike.query.common import getScopeForLine
from bike.parsing.parserutils import generateLogicalLines, \
                makeLineParseable, maskStringsAndRemoveComments
from parser import ParserError
from bike.parsing.fastparserast import Class
from bike.parsing.undo import getUndoStack
TABSIZE = 4

class coords:
    def __init__(self, line, column):
        self.column = column
        self.line = line
    def __str__(self):
        return "("+str(self.column)+","+str(self.line)+")"

commentRE = re.compile(r"#.*?$")


def extractMethod(sourceobj, startcoords, endcoords, newname):
    ExtractMethod(sourceobj, startcoords, endcoords, newname).execute()

class ExtractMethod(object):
    def __init__(self,sourcenode, startcoords, endcoords, newname):
        self.sourcenode = sourcenode

        startcoords, endcoords = \
               self._reverseCoordsIfWrongWayRound(startcoords,endcoords)

        self.startline = startcoords.line
        self.endline = endcoords.line
        self.startcol = startcoords.column
        self.endcol= endcoords.column
        self.newname = newname

        self.getLineSeperator()
        self.adjustStartColumnIfLessThanTabwidth()
        self.adjustEndColumnIfStartsANewLine()
        self.getFunctionObject()
        self.getRegionToBuffer()
        #print "-"*80
        #print self.extractedLines
        #print "-"*80
        self.deduceIfIsExpressionOrBlock()
        self.deduceIfIsMethodOrFunction()

    def getLineSeperator(self):
        startline = self.sourcenode.getLines()[self.startline-1]
        if startline.endswith("\r\n"):
            self.linesep = "\r\n"          # windoze
        else:
            self.linesep = startline[-1]   # mac or unix
        

    def adjustStartColumnIfLessThanTabwidth(self):
        tabwidth = self.getTabWidthOfLine(self.sourcenode.getLines()[self.startline-1])
        if self.startcol < tabwidth: self.startcol = tabwidth
            
    def adjustEndColumnIfStartsANewLine(self):
        if self.endcol == 0:
            self.endline -=1
            nlSize = len(self.linesep)
            self.endcol = len(self.sourcenode.getLines()[self.endline-1])-nlSize

    def execute(self):
        self.deduceArguments()
        getUndoStack().addSource(self.sourcenode,self.sourcenode.getSource())
        self.writeNewFunctionToSourceNode()
        self.addCallToNewFunction()
        self.sourcenode.setModified()

    def getFunctionObject(self):
        self.fn = getScopeForLine(self.sourcenode,self.startline)


    def getTabwidthOfParentFunction(self):
        line = self.sourcenode.getLines()[self.fn.getStartLine()-1]
        match = re.match("\s+",line)
        if match is None:
            return 0
        else:
            return match.end(0)

    # should be in the transformer module
    def writeNewFunctionToSourceNode(self):
        tabwidth = self.getTabwidthOfParentFunction()
        insertpos = self.fn.getEndLine() - 1
        
        lines = self.sourcenode.getLines()
        
        while re.match("\s*"+self.linesep,lines[insertpos-1]):
            insertpos -= 1
        
        lines.insert(insertpos, self.linesep)
        insertpos +=1

        fndefn = "def "+self.newname+"("

        if self.isAMethod:
            fndefn += "self"
            if self.fnargs != []:
                fndefn += ","+",".join(self.fnargs)
        else:
            fndefn += ",".join(self.fnargs)

        fndefn += "):"+self.linesep


        lines.insert(insertpos,tabwidth*" "+fndefn)
        insertpos +=1

        tabwidth += TABSIZE

        if self.isExpression:
            assert len(self.extractedLines) == 1
            lines.insert(insertpos,tabwidth*" "+ "return "+self.extractedLines[0])
            insertpos +=1
        else:
            for line in self.extractedLines:
                lines.insert(insertpos,tabwidth*" "+line)
                insertpos +=1
            if self.retvals != []:
                lines.insert(insertpos,tabwidth*" "+"return "+
                             ",".join(self.retvals) + self.linesep)
        #print "---- after new function insert","-"*40
        #for line in lines:
        #    print [line]
        #print "-"*80
        

    def addCallToNewFunction(self):
        lines = self.sourcenode.getLines()
        startline = self.startline
        endline = self.endline
        startcol = self.startcol
        endcol= self.endcol
        fncall = self.newname + "("+",".join(self.fnargs)+")"
        if self.isAMethod:
            fncall = "self." + fncall

        if self.retvals != []:
            fncall = ",".join(self.retvals) + " = "+fncall
        
        if startline == endline:  # i.e. extracted code part of existing line
            lines[startline-1] = lines[startline-1][:startcol] + fncall + lines[startline-1][endcol:]
            if not lines[startline-1].endswith(self.linesep):
                lines[startline-1]+=self.linesep
        else:
            tabwidth = self.getTabWidthOfLine(lines[startline-1])
            lines[startline-1] = tabwidth*" " + fncall+self.linesep
            for i in range(startline,endline):
                del lines[startline]
        #print "---- after call insert","-"*40
        #for line in lines:
        #    print [line]
        #print "-"*80
                

    def getTabWidthOfLine(self,line):
        match = re.match("\s+",line)
        if match is None:
            return 0
        else:
            return match.end(0)
        

    def deduceArguments(self):
        maskedlines = self.fn.getLinesNotIncludingThoseBelongingToChildScopes()

        # strip off comments
        lines = [commentRE.sub(self.linesep,line) for line in maskedlines]
        extractedLines = maskStringsAndRemoveComments("".join(self.extractedLines)).splitlines(1)
        
        linesbefore = lines[:(self.startline - self.fn.getStartLine())]
        linesafter = lines[(self.endline - self.fn.getStartLine()) + 1:]

        # split into logical lines
        linesbefore = [line for line in generateLogicalLines(linesbefore)]        
        extractedLines = [line for line in generateLogicalLines(extractedLines)]
        linesafter = [line for line in generateLogicalLines(linesafter)]
                
        if self.startline == self.endline:
            # need to include the line code is extracted from
            line = generateLogicalLines(lines[self.startline - self.fn.getStartLine():]).next()
            linesbefore.append(line[:self.startcol] + "dummyFn()" + line[self.endcol:])
        assigns = self.getAssignments(linesbefore)
        fnargs = self.getFunctionArgs(linesbefore)
        candidateArgs = assigns + fnargs            
        refs = self.getReferences(extractedLines)
        self.fnargs = [ref for ref in refs if ref in candidateArgs]
        
        assignsInExtractedBlock = self.getAssignments(extractedLines)
        usesAfterNewFunctionCall = self.getReferences(linesafter)
        usesInPreceedingLoop = self.getReferences(
            self.getPreceedingLinesInLoop(linesbefore,line))
        self.retvals = [ref for ref in usesInPreceedingLoop+usesAfterNewFunctionCall
                                   if ref in assignsInExtractedBlock]

    def getPreceedingLinesInLoop(self,linesbefore,firstLineToExtract):
        tabwidth = self.getTabWidthOfLine(firstLineToExtract)
        rootTabwidth = self.getTabWidthOfLine(linesbefore[0])
        llines = [line for line in generateLogicalLines(linesbefore)]
        startpos = len(llines)-1
        loopTabwidth = tabwidth
        for idx in range(startpos,0,-1):
            line = llines[idx]
            if re.match("(\s+)for",line) is not None or \
               re.match("(\s+)while",line) is not None:
                candidateLoopTabwidth = self.getTabWidthOfLine(line)
                if candidateLoopTabwidth < loopTabwidth:
                    startpos = idx
        return llines[startpos:]

    # lines = list of lines. Have to have strings masked and comments removed
    def getAssignments(self,lines):
        class AssignVisitor:
            def __init__(self):
                self.assigns = []
                
            def visitAssTuple(self, node):
                for a in node.nodes:
                    if a.name not in self.assigns:
                        self.assigns.append(a.name)

            def visitAssName(self, node):
                if node.name not in self.assigns:
                    self.assigns.append(node.name)

            def visitAugAssign(self, node):
                if isinstance(node.node, compiler.ast.Name):
                    if node.node.name not in self.assigns:
                        self.assigns.append(node.node.name)

        assignfinder = AssignVisitor()
        for line in lines:
            doctoredline = makeLineParseable(line)
            try:
                ast = compiler.parse(doctoredline)
            except ParserError:
                raise "couldnt parse:"+doctoredline
            visitor.walk(ast, assignfinder)
        return assignfinder.assigns


    # lines = list of lines. Have to have strings masked and comments removed
    def getFunctionArgs(self,lines):
        class FunctionVisitor:
            def __init__(self):
                self.result = []
            def visitFunction(self, node):
                for n in node.argnames:
                    if n != "self":
                        self.result.append(n)
        fndef = generateLogicalLines(lines).next()
        doctoredline = makeLineParseable(fndef)
        try:
            ast = compiler.parse(doctoredline)
        except ParserError:
            raise "couldnt parse:"+doctoredline
        return visitor.walk(ast, FunctionVisitor()).result

    # lines = list of lines. Have to have strings masked and comments removed
    def getReferences(self,lines):
        class NameVisitor:
            def __init__(self):
                self.result = []
            def visitName(self, node):
                if node.name not in self.result:
                    self.result.append(node.name)
        reffinder = NameVisitor()
        for line in lines:
            doctoredline = makeLineParseable(line)
            try:
                ast = compiler.parse(doctoredline)
            except ParserError:
                raise "couldnt parse:"+doctoredline
            visitor.walk(ast, reffinder)
        return reffinder.result
        
    def getRegionToBuffer(self):
        startline = self.startline
        endline = self.endline
        startcol = self.startcol
        endcol= self.endcol

        self.extractedLines = self.sourcenode.getLines()[startline-1:endline]
        
        match = re.match("\s*",self.extractedLines[0])
        tabwidth = match.end(0)

        self.extractedLines = [line[startcol:] for line in self.extractedLines]

        if startline == endline:
            # need to crop the end
            # (n.b. if region is multiple lines, then whole lines are taken)
            self.extractedLines[-1] = self.extractedLines[-1][:endcol-startcol]

        if self.extractedLines[-1] == "":
            del self.extractedLines[-1]

        if self.extractedLines[-1][-1] != '\n':
            self.extractedLines[-1] += self.linesep

    def deduceIfIsExpressionOrBlock(self):
        self.isExpression = 0
        if len(self.extractedLines) == 1:
            lines = self.sourcenode.getLines()
            linebefore = lines[self.startline-1][:self.startcol]
            if re.match("^\s*$",linebefore) is not None:
                return
            if re.search(":\s*$",linebefore) is not None:
                return
            self.isExpression = 1

    def deduceIfIsMethodOrFunction(self):
        if isinstance(self.fn.getParent(),Class):
            self.isAMethod = 1
        else:
            self.isAMethod = 0


    def _reverseCoordsIfWrongWayRound(self,startcoords,endcoords):
        if(startcoords.line > endcoords.line) or \
             (startcoords.line == endcoords.line and \
              startcoords.column > endcoords.column):
            return endcoords,startcoords
        else:
            return startcoords,endcoords

