#!/usr/bin/env python
import setpath
import unittest
from renameFunction import*
from bike import testdata
from bike.testutils import*
from bike.parsing.extended_ast import Root
import compiler

class RenameFunctionTests:
    def runTarget(self, src, klassfqn, newname):
        # see concrete subclasses for implementation
        pass

    def testRenamesFunctionDcl(self):
        srcBefore=trimLines("""
        def theFunction():
            pass
        """)
        srcAfter=trimLines("""
        def newName():
            pass
        """)
        src = self.renameFunction(srcBefore, "theFunction", "newName")
        self.assertEqual(srcAfter,src)

    def testDoesntBarfWhenFunctionIncludesBrackettedExpression(self):
        srcBefore=trimLines("""
        def theFunction():
            return ('\\n').strip()
        """)
        srcAfter=trimLines("""
        def newName():
            return ('\\n').strip()
        """)
        src = self.renameFunction(srcBefore, "theFunction", "newName")
        self.assertEqual(srcAfter,src)




class RenameFunctionTests_importsFunction:

    def testRenamesImportedFunctionReference(self):
        srcBefore=trimLines("""
        import b.bah.theFunction
        b.bah.theFunction()
        """)
        srcAfter=trimLines("""
        import b.bah.newName
        b.bah.newName()
        """)
        src = self.renameFunction(srcBefore, "a.b.bah.theFunction", "newName")
        self.assertEqual(srcAfter,src)

    def testRenamesFunctionReferenceImportedWithFromClause(self):
        srcBefore=trimLines("""
        from b.bah import theFunction
        theFunction()
        """)
        srcAfter=trimLines("""
        from b.bah import newName
        newName()
        """)
        src = self.renameFunction(srcBefore, "a.b.bah.theFunction", "newName")
        self.assertEqual(srcAfter,src)

    def testRenamesFunctionRefInImportClause(self):
        srcBefore=trimLines("""
        import b.bah.theFunction
        b.bah.theFunction()
        """)
        srcAfter=trimLines("""
        import b.bah.newName
        b.bah.newName()
        """)
        src = self.renameFunction(srcBefore, "a.b.bah.theFunction", "newName")
        self.assertEqual(srcAfter,src)

    def testRenamesFunctionRefInImportClauseWhenItsTheSecondImportInTheStatement(self):
        srcBefore=trimLines("""
        import foo.yadayada, b.bah.theFunction
        """)
        srcAfter=trimLines("""
        import foo.yadayada, b.bah.newName
        """)
        src = self.renameFunction(srcBefore, "a.b.bah.theFunction", "newName")
        self.assertEqual(srcAfter,src)

    def testRenamesFunctionRefInImportFromClause(self):
        srcBefore=trimLines("""
        from b.bah import theFunction
        theFunction()
        """)
        srcAfter=trimLines("""
        from b.bah import newName
        newName()
        """)
        src = self.renameFunction(srcBefore, "a.b.bah.theFunction", "newName")
        self.assertEqual(srcAfter,src)



class TestRenameFunction(unittest.TestCase, RenameFunctionTests):
    def renameFunction(self, src, fqn, newname):
        ast = createSourceNodeAt(Root(), src, "foo")
        renameFunction(ast, "foo."+fqn, newname)
        return getASTNode(ast, "Source").getSource()

class TestRenameFunctionReferenceWithDirectoryStructure(unittest.TestCase, RenameFunctionTests_importsFunction):

    def renameFunction(self, src, functionfqn, newname):
        ast = Root()
        ast = createSourceNodeAt(ast, src, "a.foo")
        ast = createSourceNodeAt(ast, testdata.Function, "a.b.bah")
        renameFunction(ast, functionfqn, newname)
        return getMatchingASTNode(ast, "Source",fqn="a.foo").getSource()


# checks that renameFunction can be done twice on the same ast
class TestRenameFunctionReferenceTwice(unittest.TestCase, RenameFunctionTests_importsFunction):

    def renameFunction(self, src, functionfqn, newname):
        ast = Root()
        ast = createSourceNodeAt(ast, src, "a.foo")
        ast = createSourceNodeAt(ast, testdata.Function, "a.b.bah")
        renameFunction(ast, functionfqn, "AnotherName")
        newfqn = fqn_rcdr(functionfqn)+".AnotherName"
        renameFunction(ast, newfqn, newname)
        return getMatchingASTNode(ast, "Source",fqn="a.foo").getSource()



if __name__ == "__main__":
    unittest.main()
